@@ -12,6 +12,7 @@ import json
 import logging
 import signal
 import sys
+import copy
 from collections import OrderedDict
 from os import path, environ
 
@@ -22,10 +23,18 @@ from mockintosh import configs
 from mockintosh.exceptions import UnrecognizedConfigFileFormat
 from mockintosh.handlers import Request, Response  # noqa: F401
 from mockintosh.methods import _detect_engine, _nostderr, _import_from
-from mockintosh.recognizers import PathRecognizer, HeadersRecognizer, QueryStringRecognizer, BodyRecognizer
+from mockintosh.recognizers import (
+    PathRecognizer,
+    HeadersRecognizer,
+    QueryStringRecognizer,
+    BodyTextRecognizer,
+    BodyUrlencodedRecognizer,
+    BodyMultipartRecognizer
+)
 from mockintosh.servers import HttpServer, TornadoImpl
+from mockintosh.performance import PerformanceProfile
 
-__version__ = "0.6.1"
+__version__ = "0.7"
 __location__ = path.abspath(path.dirname(__file__))
 
 should_cov = environ.get('COVERAGE_PROCESS_START', False)
@@ -43,9 +52,12 @@ class Definition():
             self.data = configs.get_default()
         else:
             self.load()
+        self.orig_data = copy.deepcopy(self.data)
         self.validate()
+        for service in self.data['services']:
+            service['orig_data'] = copy.deepcopy(service)
         self.template_engine = _detect_engine(self.data, 'config')
-        self.analyze()
+        self.data = Definition.analyze(self.data, self.template_engine)
 
     def load(self):
         if self.source_text is None:
@@ -68,48 +80,99 @@ class Definition():
         validate(instance=self.data, schema=self.schema)
         logging.info('Configuration file is valid according to the JSON schema.')
 
-    def analyze(self):
-        for service in self.data['services']:
+    @staticmethod
+    def analyze(data, template_engine):
+        if 'performanceProfiles' in data:
+            for key, performance_profile in data['performanceProfiles'].items():
+                ratio = performance_profile.get('ratio')
+                delay = performance_profile.get('delay', 0.0)
+                faults = performance_profile.get('faults', {})
+                data['performanceProfiles'][key] = PerformanceProfile(ratio, delay=delay, faults=faults)
+        else:
+            data['performanceProfiles'] = {}
+
+        global_performance_profile = None
+        if 'globals' in data:
+            global_performance_profile = data['globals'].get('performanceProfile', None)
+
+        for service in data['services']:
             if 'endpoints' not in service:
                 continue
-            for endpoint in service['endpoints']:
-                endpoint['params'] = {}
-                endpoint['context'] = OrderedDict()
+            service = Definition.analyze_service(
+                service,
+                template_engine,
+                performance_profiles=data['performanceProfiles'],
+                global_performance_profile=global_performance_profile
+            )
+        return data
 
-                path_recognizer = PathRecognizer(
-                    endpoint['path'],
+    @staticmethod
+    def analyze_service(service, template_engine, performance_profiles={}, global_performance_profile=None):
+        service_perfomance_profile = service.get('performanceProfile', global_performance_profile)
+        for endpoint in service['endpoints']:
+            endpoint['internalOrigPath'] = endpoint['path']
+            endpoint['params'] = {}
+            endpoint['context'] = OrderedDict()
+            endpoint['performanceProfile'] = performance_profiles.get(
+                endpoint.get('performanceProfile', service_perfomance_profile),
+                None
+            )
+
+            path_recognizer = PathRecognizer(
+                endpoint['path'],
+                endpoint['params'],
+                endpoint['context'],
+                template_engine
+            )
+            endpoint['path'], endpoint['priority'] = path_recognizer.recognize()
+
+            if 'headers' in endpoint and endpoint['headers']:
+                headers_recognizer = HeadersRecognizer(
+                    endpoint['headers'],
                     endpoint['params'],
                     endpoint['context'],
-                    self.template_engine
+                    template_engine
                 )
-                endpoint['path'], endpoint['priority'] = path_recognizer.recognize()
+                endpoint['headers'] = headers_recognizer.recognize()
 
-                if 'headers' in endpoint and endpoint['headers']:
-                    headers_recognizer = HeadersRecognizer(
-                        endpoint['headers'],
-                        endpoint['params'],
-                        endpoint['context'],
-                        self.template_engine
-                    )
-                    endpoint['headers'] = headers_recognizer.recognize()
+            if 'queryString' in endpoint and endpoint['queryString']:
+                headers_recognizer = QueryStringRecognizer(
+                    endpoint['queryString'],
+                    endpoint['params'],
+                    endpoint['context'],
+                    template_engine
+                )
+                endpoint['queryString'] = headers_recognizer.recognize()
 
-                if 'queryString' in endpoint and endpoint['queryString']:
-                    headers_recognizer = QueryStringRecognizer(
-                        endpoint['queryString'],
-                        endpoint['params'],
-                        endpoint['context'],
-                        self.template_engine
-                    )
-                    endpoint['queryString'] = headers_recognizer.recognize()
-
-                if 'body' in endpoint and 'text' in endpoint['body'] and endpoint['body']['text']:
-                    body_recognizer = BodyRecognizer(
+            if 'body' in endpoint:
+                if 'text' in endpoint['body'] and endpoint['body']['text']:
+                    body_text_recognizer = BodyTextRecognizer(
                         endpoint['body']['text'],
                         endpoint['params'],
                         endpoint['context'],
-                        self.template_engine
+                        template_engine
                     )
-                    endpoint['body']['text'] = body_recognizer.recognize()
+                    endpoint['body']['text'] = body_text_recognizer.recognize()
+
+                if 'urlencoded' in endpoint['body'] and endpoint['body']['urlencoded']:
+                    body_urlencoded_recognizer = BodyUrlencodedRecognizer(
+                        endpoint['body']['urlencoded'],
+                        endpoint['params'],
+                        endpoint['context'],
+                        template_engine
+                    )
+                    endpoint['body']['urlencoded'] = body_urlencoded_recognizer.recognize()
+
+                if 'multipart' in endpoint['body'] and endpoint['body']['multipart']:
+                    body_multipart_recognizer = BodyMultipartRecognizer(
+                        endpoint['body']['multipart'],
+                        endpoint['params'],
+                        endpoint['context'],
+                        template_engine
+                    )
+                    endpoint['body']['multipart'] = body_multipart_recognizer.recognize()
+
+        return service
 
 
 def get_schema():
